"""Enum of valid terminal styles and colours."""
from __future__ import annotations

from enum import Enum


class Styles(str, Enum):
    """
    Enum of valid terminal styles and colours.

    Special styles:
        - END: Resets all styles and colours.
        - BOLD: Bold text.
        - DIM: Dimmed text.
        - ITALIC: Italic text.
        - UNDERLINE: Underlined text.
    """

    END = "\033[0m"
    BOLD = "\033[1m"
    DIM = "\033[2m"
    ITALIC = "\033[3m"
    UNDERLINE = "\033[4m"

    BLACK = "\033[30m"
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    MAGENTA = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"

    LIGHT_BLACK = "\033[90m"
    LIGHT_RED = "\033[91m"
    LIGHT_GREEN = "\033[92m"
    LIGHT_YELLOW = "\033[93m"
    LIGHT_BLUE = "\033[94m"
    LIGHT_MAGENTA = "\033[95m"
    LIGHT_CYAN = "\033[96m"
    LIGHT_WHITE = "\033[97m"

    BG_BLACK = "\033[40m"
    BG_RED = "\033[41m"
    BG_GREEN = "\033[42m"
    BG_YELLOW = "\033[43m"
    BG_BLUE = "\033[44m"
    BG_MAGENTA = "\033[45m"
    BG_CYAN = "\033[46m"
    BG_WHITE = "\033[47m"

    BG_LIGHT_BLACK = "\033[100m"
    BG_LIGHT_RED = "\033[101m"
    BG_LIGHT_GREEN = "\033[102m"
    BG_LIGHT_YELLOW = "\033[103m"
    BG_LIGHT_BLUE = "\033[104m"
    BG_LIGHT_MAGENTA = "\033[105m"
    BG_LIGHT_CYAN = "\033[106m"
    BG_LIGHT_WHITE = "\033[107m"



"""Print to stdout with colour."""
from __future__ import annotations

from typing import List
from typing import Optional
from typing import Union

from colourout.process_styles import process_styles
from colourout.styles import Styles


def printc(
    *__values: object,
    sep: str = " ",
    end: str = "\n",
    styles: Optional[List[Union[Styles, str]]] = None,
) -> None:
    r"""
    Print with colour.

    Prints text to stdout with provided formatting.
    Always ends with a reset.

    Params
    ------
    __values: :class:`object`
        The values to print.

    sep: :class:`str`
        The separator to use between values.

    end: :class:`str`
        The string to print at the end.

    styles: :class:`List[Union[Styles, str]]`
        The styles to apply to the output.

    >>> printc("Hello, world!", styles=["BLUE", "ITALIC"])
    \033[94m\033[3mHello, world!\033[0m
    """
    styles = styles or [Styles.END]
    processed_styles = process_styles(styles)

    print(*processed_styles, sep="", end="")
    print(*__values, sep=sep, end=end)
    print(Styles.END.value, end="")



"""Process styles from strings to Enum values."""
from __future__ import annotations

from typing import List
from typing import Union

from colourout.styles import Styles


def process_styles(styles: List[Union[Styles, str]]) -> List[str]:
    """Convert strings to Enum values."""
    processed = []

    for style in styles:
        if not isinstance(style, Styles):
            style = Styles[style]

        processed.append(style.value)

    return processed



"""Setup script for the colourout package."""
from __future__ import annotations

import setuptools

import colourout

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="colourout",
    version=colourout.__version__,
    description="A Python package for printing coloured text to the terminal.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/sam-kenney/colourout",
    author="Sam Kenney",
    author_email="sam.kenney@me.com",
    license="MIT",
    packages=["colourout"]
    + [f"colourout.{sub}" for sub in setuptools.find_packages("colourout")],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



r"""
Colourout - A Python package for printing coloured text to the terminal.

Exports
-------
:meth:`printc`
    Print coloured text to stdout.

:meth:`eprintc`
    Print coloured text to stderr.

:class:`Styles`
    Enum of valid terminal styles, foreground colours, and background colours.
    Background colours are prefixed with `BG_`.
    Light colours are prefixed with `LIGHT_`.
    Light colours may not be supported by all terminals.

Examples
--------
>>> from colourout import printc
>>> printc("Hello, world!", styles=["BLUE", "ITALIC"])
\033[94m\033[3mHello, world!\033[0m
>>> from colourout import eprintc
>>> eprintc("Hello, world!", styles=["BLUE", "ITALIC"])
\033[94m\033[3mHello, world!\033[0m
>>> from colourout import Styles
>>> printc("Hello, world!", styles=[Styles.BLUE, Styles.ITALIC])
\033[94m\033[3mHello, world!\033[0m
>>> printc("Hello, world!", styles=[Styles.BLUE, "ITALIC"])
\033[94m\033[3mHello, world!\033[0m
"""
from __future__ import annotations

__all__ = ("eprintc", "printc", "Styles")
__version__ = "0.1.0"

from colourout.eprintc import eprintc
from colourout.printc import printc
from colourout.styles import Styles
